-- wengwengweng

local Class = require("lib/vendor/classic")
local Timer = require("lib/components/timer")
local Sprite = require("lib/components/sprite")
local Props = require("lib/core/props")
local Animation = Class:extend()

Animation:implement(Props.Pos)
Animation:implement(Props.Size)
Animation:implement(Props.Scale)
Animation:implement(Props.Offset)
Animation:implement(Props.Rot)
Animation:implement(Props.Color)
Animation:implement(Sprite)

function Animation:new()

	self:setPos(0, 0)
	self:setScale(1, 1)
	self:setOffset(0, 0)
	self:setRot(0)
	self:setColor({ 255, 255, 255, 255 })
	self.anims = {}

end

function Animation:add(name, spritesheet, frameCount, speed)

	self.anims[name] = {

		name = name,
		spritesheet = spritesheet,
		frameCount = frameCount,
		speed = speed,
		frame = 1,
		frames = {},
		looping = false,
		timer = nil,

		cbs = {

			every = function() end,
			after = function() end

		}

	}

	return self

end

function Animation:play(name)

	self.current = self.anims[name]
	self:goto(1)
	self:setImg(self.current.spritesheet)

	for i = 1, self.current.frameCount do

		self.current.frames[i] = love.graphics.newQuad((i - 1) * self.img:getWidth() / self.current.frameCount, 0, self.img:getWidth() / self.current.frameCount, self.img:getHeight(), self.img:getDimensions())

	end

	return self:init()

end

function Animation:goto(f)

	self.current.frame = f

	return self

end

function Animation:init()

	self.current.timer = Timer.every(self.current.speed / self.current.frameCount, function()

		if (self.current.frame >= #self.current.frames) then

			if (self.current.looping) then

				self.current.frame = 1
				self.current.cbs.every()

			else

				self:stop()
				self.current.cbs.after()

			end

		else

			self.current.frame = self.current.frame + 1

		end

	end)

	return self

end

function Animation:setSpeed(s)

	self.current.speed = s

	return self

end

function Animation:start()

	self.current.timer:start()

	return self

end

function Animation:stop()

	self.current.timer:stop()

	return self

end

function Animation:loop()

	self.current.looping = true

	return self

end

function Animation:unloop()

	self.current.looping = false

	return self

end

function Animation:every(f)

	self.current.cbs.every = f or self.current.cbs.every or function() end

	return self

end

function Animation:after(f)

	self.current.cbs.after = f or self.current.cbs.after or function() end

	return self

end

function Animation:update(dt)

	self.current.timer:update(dt)

end

function Animation:draw()

	love.graphics.setColor(self.color)

	love.graphics.draw(self.img,
				   self.current.frames[self.current.frame],
				   self.pos.x,
				   self.pos.y,
				   self.rot,
				   self.size.width / (self.img:getWidth() / #self.current.frames) * self.scale.x,
				   self.size.height / self.img:getHeight() * self.scale.y,
				   self.offset.x * (self.img:getWidth() / #self.current.frames),
				   self.offset.y * self.img:getHeight())

	love.graphics.setColor({ 255, 255, 255, 255 })

end

function Animation:setWidth(width)

	return self:setSize(width, width * self.img:getHeight() / (self.img:getWidth() / #self.current.frames))

end

function Animation:setHeight(height)

	return self:setSize(height * (self.img:getWidth() / #self.current.frames) / self.img:getHeight(), height)

end

return Animation
