-- wengwengweng

local Class = require("lib/vendor/classic")
local Shake = Class:extend()

function Shake:new(w, h)

	self:setSize(w, h)
	self:setSpeed()
	self:init()

end

function Shake:init()

	self.props = {

		shake = 0,
		rotate = 0,
		shear = { x = 0, y = 0 },
		scale = { x = 1, y = 1 }

	}

	return self

end

function Shake:setSize(w, h)

	self.width = w or love.graphics.getWidth()
	self.height = h or love.graphics.getHeight()

	return self

end

function Shake:setSpeed(s)

	self.speed = s or 7

	return self

end

local function lerp(a, b, k)

	if (a == b) then

		return a

	else

		if (math.abs(a - b) < 0.005) then

			return b

		else

			return a * (1 - k) + b * k end

	end

end

function Shake:update(dt)

	self.props.shake = lerp(self.props.shake, 0, self.speed * dt)
	self.props.rotate = lerp(self.props.rotate, 0, self.speed * dt)
	self.props.scale.x = lerp(self.props.scale.x, 1, self.speed * dt)
	self.props.scale.y = lerp(self.props.scale.y, 1, self.speed * dt)
	self.props.shear.x = lerp(self.props.shear.x, 0, self.speed * dt)
	self.props.shear.y = lerp(self.props.shear.y, 0, self.speed * dt)

end

function Shake:draw()

	love.graphics.translate(self.width * .5, self.height * .5)
	love.graphics.rotate((math.random() - .5) * self.props.rotate)
	love.graphics.scale(self.props.scale.x, self.props.scale.y)
	love.graphics.translate(-self.width * .5, -self.height * .5)
	love.graphics.translate((math.random() - .5) * self.props.shake, (math.random()-.5) * self.props.shake)
	love.graphics.shear(self.props.shear.x * .01, self.props.shear.y * .01)

end

function Shake:shake(r)

	self.props.shake = r

	return self

end

function Shake:rotate(r)

	self.props.rotate = r

	return self

end

function Shake:scale(x, y)

	if (not y) then

		return self:scale(x, x)

	else

		self.props.scale.x = x
		self.props.scale.y = y

	end

	return self

end

function Shake:shear(x, y)

	self.props.shear.x = x
	self.props.shear.y = y

	return self

end

return Shake
