-- wengwengweng

local gamera = require("lib/vendor/gamera")
local mouse = require("lib/core/mouse")
local Audio = require("lib/components/audio")
local Timer = require("lib/components/timer")
local Hand = require("lib/entities/hand")
local Pimple = require("lib/entities/pimple")
local Face = require("lib/entities/face")
local Conjun = require("lib/entities/conjun")
local Poojoobb = require("lib/entities/poojoobb")
local Bumgo = require("lib/entities/bumgo")
local Squeeze = {}

function Squeeze:enter()

	self.face = Face(0, 0)
	self.cam = gamera.new(0, 0, self.face.size.width, self.face.size.height)
	self.cam:setWindow(32, 32, 640, conf.height - 64)
	self.music = Audio(assets.audio.bumgo):setLooping(true):play()
	self.hand = Hand()
	self.organs = {}
	self.pimples = {}
	self.timers = {}
	self.spawnSpeed = 6
	self.camSpeed = 160
	self.popped = 0

	self.widgets = {

		conjun = Conjun(712, 80),
		poojoobb = Poojoobb(712, 252),
		bumgo = Bumgo(712, 424)

	}

	self.timers.communicate = Timer.every(love.math.random(8), function()

		self:communicate()
		self.timers.communicate:setTime(love.math.random(8))

	end)

end

function Squeeze:update(dt)

	self.hand:setPos(self.cam:toWorld(mouse.x(), mouse.y()))
	self:browse(dt)
	self.face:update(dt)
	for k, pimple in ipairs(self.pimples) do pimple:update(dt) end
	for k, widget in pairs(self.widgets) do widget:update(dt) end
	for k, timer in pairs(self.timers) do timer:update(dt) end
	self:grow(dt)

end

function Squeeze:draw()

	self.cam:push()
	self.face:draw()
	for k, pimple in ipairs(self.pimples) do pimple:draw() end
	self.hand:draw()
	self.cam:pop()
	self.cam:drawBound()
	for k, widget in pairs(self.widgets) do widget:draw() end

end

function Squeeze:browse(dt)

	-- navigate through face
	-- adjust the hand

	local up = love.keyboard.isDown("w") or love.keyboard.isDown("up")
	local left = love.keyboard.isDown("a") or love.keyboard.isDown("left")
	local down = love.keyboard.isDown("s") or love.keyboard.isDown("down")
	local right = love.keyboard.isDown("d") or love.keyboard.isDown("right")
	local camDx, camDy, camX, camY
	local windowX, windowY, windowWidth, windowHeight = self.cam:getWindow()
	local camMinX = windowWidth / 2
	local camMinY = windowHeight / 2
	local camMaxX = self.face.size.width - camMinX
	local camMaxY = self.face.size.height - camMinY

	if (up or left or down or right) then

		if (up) then camDx, camDy = (camDx or 0), -self.camSpeed * dt
		elseif (down) then camDx, camDy = (camDx or 0), self.camSpeed * dt end
		if (left) then camDx, camDy = -self.camSpeed * dt, (camDy or 0)
		elseif (right) then camDx, camDy = self.camSpeed * dt, (camDy or 0) end

		self.cam:move(camDx, camDy)
		camX, camY = self.cam:getPosition()
		self.hand:move(((camX == camMinX or camX == camMaxX) and 0 or camDx), ((camY == camMinY or camY == camMaxY) and 0 or camDy))

	end

	-- mouse navigation

	-- local mouseX = mouse.x() - windowX
	-- local mouseY = mouse.y() - windowY
	-- local camDx, camDy
	-- local s = 172

	-- if (mouseX < s or mouseX > windowWidth - s or mouseY < s or mouseY > windowHeight - s) then

	-- 	if (mouseX < s) then camDx, camDy = -self.camSpeed * dt, (camDy or 0)
	-- 	elseif (mouseX > windowWidth - s) then camDx, camDy = self.camSpeed * dt, (camDy or 0) end
	-- 	if (mouseY < s) then camDx, camDy = (camDx or 0), -self.camSpeed * dt
	-- 	elseif (mouseY > windowHeight - s) then camDx, camDy = (camDx or 0), self.camSpeed * dt end

	-- 	self.cam:move(camDx, camDy)

	-- end

end

function Squeeze:spawn()

	-- spawn a pimple in free area

	local padding = 32
	local x = love.math.random(padding, self.face.size.width - padding)
	local y = love.math.random(padding, self.face.size.height - padding)

	for k, organ in ipairs(self.face.organs) do

		if organ:focused(x, y) then

			return self:spawn()

		end

	end

	local p = Pimple(x, y, 48)

	p.sound:setVolume(0.1)
	self.pimples[#self.pimples + 1] = p

end

function Squeeze:grow()

	-- grow pimples due to the pimple popped

	if (love.math.random(self.spawnSpeed + self.popped) == 1) then

		self:spawn()

	end

end

function Squeeze:release()

	-- release hand

	self.hand:release()

end

function Squeeze:squeeze()

	-- squeeze hand
	-- pop the squeezed pimple

	self.hand:squeeze()

	for k, pimple in ipairs(self.pimples) do

		if (not pimple.popped and pimple:focused(self.cam:toWorld(mouse.x(), mouse.y()))) then

			self:pop(pimple)

		end

	end

end

function Squeeze:communicate()

	local coms = {

		assets.audio.com1,
		assets.audio.com2,
		assets.audio.com3,
		assets.audio.com4,
		assets.audio.com5

	}

	Audio(coms[love.math.random(#coms)]):setVolume(0.3):setPitch(0.8):play()

end

function Squeeze:pop(pimple)

	self.popped = self.popped + 1
	pimple:pop(self.face.size.height)
	self.widgets.bumgo:shake()

end

function Squeeze:leave()

	self.music:stop()

end

function Squeeze:mousepressed()

	self:squeeze()

end

function Squeeze:mousereleased()

	self:release()

end

function Squeeze:keypressed(key)

	if (key == "space") then

		self:squeeze()

	end

end

function Squeeze:keyreleased(key)

	self:release()

end

return Squeeze
